@@ -14,7 +14,7 @@ with:
     - "grid_path": the path where the powergrid has been loaded from
 
   - "episode_times.json": gives some information about the total time spend in multiple part of the runner, mainly the
-    :class:`grid2op.Agent` (and especially its method :func:`grid2op.Agent.act`) and amount of time spent in the
+    :class:`grid2op.BaseAgent` (and especially its method :func:`grid2op.BaseAgent.act`) and amount of time spent in the
     :class:`grid2op.Environment`
 
   - "_parameters.json": is a representation as json of a the :class:`grid2op.Parameters.Parameters` used for this episode
@@ -22,19 +22,19 @@ with:
     reward at index `i` is the one observed by the agent at time `i` and **NOT** the reward sent by the
     :class:`grid2op.Environment` after the action has been implemented.
   - "exec_times.npy" is a numpy 1d array giving the execution time of each time step of the episode
-  - "actions.npy" gives the actions that has been taken by the :class:`grid2op.Agent.Agent`. At row `i` of "actions.npy" is a
+  - "actions.npy" gives the actions that has been taken by the :class:`grid2op.BaseAgent.BaseAgent`. At row `i` of "actions.npy" is a
     vectorized representation of the action performed by the agent at timestep `i` *ie.* **after** having observed
     the observation present at row `i` of "observation.npy" and the reward showed in row `i` of "rewards.npy".
   - "disc_lines.npy" gives which lines have been disconnected during the simulation of the cascading failure at each
     time step. The same convention as for "rewards.npy" has been adopted. This means that the powerlines are
-    disconnected when the :class:`grid2op.Agent` takes the :class:`grid2op.Action` at time step `i`.
-  - "observations.npy" is a numpy 2d array reprensenting the :class:`grid2op.Observation.Observation` at the disposal of the
-    :class:`grid2op.Agent` when he took his action.
+    disconnected when the :class:`grid2op.BaseAgent` takes the :class:`grid2op.BaseAction` at time step `i`.
+  - "observations.npy" is a numpy 2d array reprensenting the :class:`grid2op.BaseObservation.BaseObservation` at the disposal of the
+    :class:`grid2op.BaseAgent` when he took his action.
   - "env_modifications.npy" is a 2d numpy array representing the modification of the powergrid from the environment.
     these modification usually concerns the hazards, maintenance, as well as modification of the generators production
     setpoint or the loads consumption.
 
-All of the above should allow to read back, and better understand the behaviour of some :class:`grid2op.Agent.Agent`, even
+All of the above should allow to read back, and better understand the behaviour of some :class:`grid2op.BaseAgent.BaseAgent`, even
 though such utility functions have not been coded yet.
 """
 
@@ -61,6 +61,7 @@ class EpisodeData:
     PARAMS = "_parameters.json"
     META = "episode_meta.json"
     TIMES = "episode_times.json"
+    OTHER_REWARDS = "other_rewards.json"
 
     AG_EXEC_TIMES = "agent_exec_times.npy"
     ACTIONS = "actions.npy"
@@ -74,7 +75,7 @@ class EpisodeData:
                  params=None, meta=None, episode_times=None,
                  observation_space=None, action_space=None,
                  helper_action_env=None, path_save=None, disc_lines_templ=None,
-                 logger=None, name=str(1), get_dataframes=None):
+                 logger=None, name=str(1), get_dataframes=None, other_rewards=[]):
 
         self.actions = CollectionWrapper(actions, action_space, "actions")
         self.observations = CollectionWrapper(observations, observation_space,
@@ -82,6 +83,7 @@ class EpisodeData:
 
         self.env_actions = CollectionWrapper(env_actions, helper_action_env,
                                              "env_actions")
+        self.other_rewards = other_rewards
         self.observation_space = observation_space
         self.helper_action_env = helper_action_env
         self.rewards = rewards
@@ -158,6 +160,8 @@ class EpisodeData:
                 episode_meta = json.load(fp=f)
             with open(os.path.join(episode_path, EpisodeData.TIMES)) as f:
                 episode_times = json.load(fp=f)
+            with open(os.path.join(episode_path, EpisodeData.OTHER_REWARDS)) as f:
+                other_rewards = json.load(fp=f)
 
             times = np.load(os.path.join(
                 episode_path, EpisodeData.AG_EXEC_TIMES))
@@ -169,6 +173,7 @@ class EpisodeData:
             disc_lines = np.load(os.path.join(
                 episode_path, EpisodeData.LINES_FAILURES))
             rewards = np.load(os.path.join(episode_path, EpisodeData.REWARDS))
+
         except FileNotFoundError as ex:
             raise Grid2OpException(f"EpisodeData file not found \n {str(ex)}")
 
@@ -182,10 +187,10 @@ class EpisodeData:
         return cls(actions, env_actions, observations, rewards, disc_lines,
                    times, _parameters, episode_meta, episode_times,
                    observation_space, action_space, helper_action_env,
-                   agent_path, name=name, get_dataframes=True)
+                   agent_path, name=name, get_dataframes=True,
+                   other_rewards=other_rewards)
 
     def set_parameters(self, env):
-
         if self.serialize:
             self.parameters = env.parameters.to_dict()
 
@@ -237,6 +242,16 @@ class EpisodeData:
                         self.disc_lines = np.concatenate(
                             (self.disc_lines, self.disc_lines_templ))
 
+            if "rewards" in info:
+                self.other_rewards.append({k: self._convert_to_float(v) for k, v in info["rewards"].items()})
+
+    def _convert_to_float(self, el):
+        try:
+            res = float(el)
+        except Exception as e:
+            res = -float('inf')
+        return res
+
     def set_episode_times(self, env, time_act, beg_, end_):
         if self.serialize:
             self.episode_times = {}
@@ -248,8 +263,8 @@ class EpisodeData:
                 env._time_powerflow)
             self.episode_times["Env"]["observation_computation"] = float(
                 env._time_extract_obs)
-            self.episode_times["Agent"] = {}
-            self.episode_times["Agent"]["total"] = float(time_act)
+            self.episode_times["BaseAgent"] = {}
+            self.episode_times["BaseAgent"]["total"] = float(time_act)
             self.episode_times["total"] = float(end_ - beg_)
 
     def to_disk(self):
@@ -269,6 +284,12 @@ class EpisodeData:
                 json.dump(obj=self.episode_times, fp=f,
                           indent=4, sort_keys=True)
 
+            episode_other_rewards_path = os.path.join(
+                self.episode_path, EpisodeData.OTHER_REWARDS)
+            with open(episode_other_rewards_path, "w") as f:
+                json.dump(obj=self.other_rewards, fp=f,
+                          indent=4, sort_keys=True)
+
             np.save(os.path.join(self.episode_path, EpisodeData.AG_EXEC_TIMES),
                     self.times)
             self.actions.save(
@@ -286,7 +307,7 @@ class EpisodeData:
 class CollectionWrapper:
     """
     A wrapping class to add some behaviors (iterability, item access, update, save)
-    to grid2op object collections (Action and Observation classes essentially).
+    to grid2op object collections (BaseAction and BaseObservation classes essentially).
 
     Attributes
     ----------
