@@ -1,6 +1,6 @@
 """
 The challenge "learning to run a power network" offers different _parameters to be customized, or to learn an
-:class:`grid2op.Agent` that will perform better for example.
+:class:`grid2op.BaseAgent` that will perform better for example.
 
 This class is an attempt to group them all inside one single structure.
 
@@ -18,7 +18,7 @@ class Parameters:
     Main classes representing the _parameters of the game. The main paratemeters are describe bellow.
 
     Note that changing the values of these _parameters might not be enough. If these _parameters are not used in the
-    :class:`grid2op.GameRules`, then modifying them will have no impact at all.
+    :class:`grid2op.RulesChecker`, then modifying them will have no impact at all.
 
     Attributes
     ----------
@@ -38,14 +38,14 @@ class Parameters:
 
     NB_TIMESTEP_LINE_STATUS_REMODIF: ``int``
         When someone acts on a powerline by changing its status (connected / disconnected) this number indicates
-        how many timesteps the :class:`grid2op.Agent.Agent` has to wait before being able to modify this status again.
-        For examle, if this is 1, this means that an Agent can act on status of a powerline 1 out of 2 time step (1
+        how many timesteps the :class:`grid2op.BaseAgent.BaseAgent` has to wait before being able to modify this status again.
+        For examle, if this is 1, this means that an BaseAgent can act on status of a powerline 1 out of 2 time step (1
         time step it acts, another one it cools down, and the next one it can act again). Having it at 0 it equivalent
         to deactivate this feature (default).
 
     NB_TIMESTEP_TOPOLOGY_REMODIF: ``int``
         When someone changes the topology of a substations, this number indicates how many timesteps the
-        :class:`grid2op.Agent.Agent` has to wait before being able to modify the topology on this same substation. It
+        :class:`grid2op.BaseAgent.BaseAgent` has to wait before being able to modify the topology on this same substation. It
         has the same behaviour as :attr:`Parameters.NB_TIMESTEP_LINE_STATUS_REMODIF`. To deactivate this feature,
         put it at 0 (default).
 
@@ -60,17 +60,17 @@ class Parameters:
         "alternative current" powerflow. It is also less precise. The default is ``False``
 
     FORECAST_DC: ``bool``
-        Whether to use the direct current approximation in the :func:`grid2op.Observation.forecasts` method. Default
+        Whether to use the direct current approximation in the :func:`grid2op.BaseObservation.forecasts` method. Default
         is ``False``. This can speed up the computation.
 
     MAX_SUB_CHANGED: ``int``
         Maximum number of substations that can be reconfigured between two consecutive timesteps by an
-        :class:`grid2op.Agent`. Default value is 1.
+        :class:`grid2op.BaseAgent`. Default value is 1.
 
 
     MAX_LINE_STATUS_CHANGED: ``int``
         Maximum number of powerlines statuses that can be changed between two consecutive timestetps by an
-        :class:`grid2op.Agent`. Default value is 1.
+        :class:`grid2op.BaseAgent`. Default value is 1.
 
     """
     def __init__(self, parameters_path=None):
@@ -79,7 +79,7 @@ class Parameters:
         Parameters
         ----------
         parameters_path: ``str``, optional
-            Path where to look for _parameters.
+            Path where to look for parameters.
         """
         # if True, then it will not disconnect lines above their thermal limits
         self.NO_OVERFLOW_DISCONNECTION = False
