@@ -1,6 +1,6 @@
 """
 This module is here to facilitate the evaluation of agent.
-It can handles all types of :class:`grid2op.Agent`.
+It can handles all types of :class:`grid2op.BaseAgent`.
 
 """
 import time
@@ -18,38 +18,21 @@ import json
 
 import pdb
 
-try:
-    from .Action import HelperAction, Action, TopologyAction
-    from .Exceptions import *
-    from .Observation import CompleteObservation, ObservationHelper, Observation
-    from .Reward import FlatReward, RewardHelper, Reward
-    from .GameRules import GameRules, AllwaysLegal, LegalAction
-    from .Environment import Environment
-    from .ChronicsHandler import ChronicsHandler, GridStateFromFile, GridValue
-    from .Backend import Backend
-    from .BackendPandaPower import PandaPowerBackend
-    from .Parameters import Parameters
-    from .Agent import DoNothingAgent, Agent
-    from .EpisodeData import EpisodeData
-    from ._utils import _FakePbar
-    from .VoltageControler import ControlVoltageFromFile
-
-except (ModuleNotFoundError, ImportError):
-    from Action import HelperAction, Action, TopologyAction
-    from Exceptions import *
-    from Observation import CompleteObservation, ObservationHelper, Observation
-    from Reward import FlatReward, RewardHelper, Reward
-    from GameRules import GameRules, AllwaysLegal, LegalAction
-    from Environment import Environment
-    from ChronicsHandler import ChronicsHandler, GridStateFromFile, GridValue
-    from Backend import Backend
-    from BackendPandaPower import PandaPowerBackend
-    from Parameters import Parameters
-    from Agent import DoNothingAgent, Agent
-    from EpisodeData import EpisodeData
-    from _utils import _FakePbar
-    from VoltageControler import ControlVoltageFromFile
-
+# from grid2op.BaseAction import HelperAction, BaseAction, TopologyAction
+from grid2op.Action import BaseAction, TopologyAction, DontAct
+from grid2op.Exceptions import *
+from grid2op.Observation import CompleteObservation, BaseObservation
+from grid2op.Reward import FlatReward, BaseReward
+from grid2op.Rules import AlwaysLegal, BaseRules
+from grid2op.Environment import Environment
+from grid2op.Chronics import ChronicsHandler, GridStateFromFile, GridValue
+from grid2op.Backend import Backend, PandaPowerBackend
+from grid2op.Parameters import Parameters
+from grid2op.Agent import DoNothingAgent, BaseAgent
+from grid2op.EpisodeData import EpisodeData
+from grid2op._utils import _FakePbar
+from grid2op.VoltageControler import ControlVoltageFromFile
+from grid2op.Opponent import BaseOpponent
 
 # TODO have a vectorized implementation of everything in case the agent is able to act on multiple environment
 # at the same time. This might require a lot of work, but would be totally worth it! (especially for Neural Net based agents)
@@ -125,7 +108,7 @@ class ConsoleLog(DoNothingLog):
 class Runner(object):
     """
     A runner is a utilitary tool that allows to create environment, and run simulations more easily.
-    This specific class as for main purpose to evaluate the performance of a trained :class:`grid2op.Agent`, rather
+    This specific class as for main purpose to evaluate the performance of a trained :class:`grid2op.BaseAgent`, rather
     than to train it. Of course, it is possible to adapt it for a specific training mechanisms. Examples of such
     will be made available in the future.
 
@@ -138,18 +121,18 @@ class Runner(object):
     actionClass: ``type``
         The type of action that can be performed by the agent / bot / controler. The class should be given, and
         **not** an instance of this class. This type
-        should derived from :class:`grid2op.Action`. The default is :class:`grid2op.TopologyAction`.
+        should derived from :class:`grid2op.BaseAction`. The default is :class:`grid2op.TopologyAction`.
 
     observationClass: ``type``
-        This type represents the class that will be used to build the :class:`grid2op.Observation` visible by the
-        :class:`grid2op.Agent`. As :attr:`Runner.actionClass`, this should be a type, and **not** and instance (object)
-        of this type. This type should derived from :class:`grid2op.Observation`. The default is
+        This type represents the class that will be used to build the :class:`grid2op.BaseObservation` visible by the
+        :class:`grid2op.BaseAgent`. As :attr:`Runner.actionClass`, this should be a type, and **not** and instance (object)
+        of this type. This type should derived from :class:`grid2op.BaseObservation`. The default is
         :class:`grid2op.CompleteObservation`.
 
     rewardClass: ``type``
-        Representes the type used to build the rewards that are given to the :class:`Agent`. As
+        Representes the type used to build the rewards that are given to the :class:`BaseAgent`. As
         :attr:`Runner.actionClass`, this should be a type, and **not** and instance (object) of this type.
-        This type should derived from :class:`grid2op.Reward`. The default is :class:`grid2op.ConstantReward` that
+        This type should derived from :class:`grid2op.BaseReward`. The default is :class:`grid2op.ConstantReward` that
         **should not** be used to train or evaluate an agent, but rather as debugging purpose.
 
     gridStateclass: ``type``
@@ -158,9 +141,9 @@ class Runner(object):
         :class:`grid2op.GridStateFromFile` and it must be a subclass of :class:`grid2op.GridValue`.
 
     legalActClass: ``type``
-        This types control the mechanisms to assess if an :class:`grid2op.Action` is legal.
+        This types control the mechanisms to assess if an :class:`grid2op.BaseAction` is legal.
         Like every "\.*Class" attributes the type should be pass and not an intance (object) of this type.
-        Its default is :class:`grid2op.AllwaysLegal` and it must be a subclass of :class:`grid2op.LegalAction`.
+        Its default is :class:`grid2op.AlwaysLegal` and it must be a subclass of :class:`grid2op.BaseRules`.
 
     backendClass: ``type``
         This types control the backend, *eg.* the software that computes the powerflows.
@@ -168,10 +151,10 @@ class Runner(object):
         Its default is :class:`grid2op.PandaPowerBackend` and it must be a subclass of :class:`grid2op.Backend`.
 
     agentClass: ``type``
-        This types control the type of Agent, *eg.* the bot / controler that will take :class:`grid2op.Action` and
+        This types control the type of BaseAgent, *eg.* the bot / controler that will take :class:`grid2op.BaseAction` and
         avoid cascading failures.
         Like every "\.*Class" attributes the type should be pass and not an intance (object) of this type.
-        Its default is :class:`grid2op.DoNothingAgent` and it must be a subclass of :class:`grid2op.Agent`.
+        Its default is :class:`grid2op.DoNothingAgent` and it must be a subclass of :class:`grid2op.BaseAgent`.
 
     logger:
         A object than can be used to log information, either in a text file, or by printing them to the command prompt.
@@ -228,7 +211,7 @@ class Runner(object):
                  actionClass=TopologyAction,
                  observationClass=CompleteObservation,
                  rewardClass=FlatReward,
-                 legalActClass=AllwaysLegal,
+                 legalActClass=AlwaysLegal,
                  envClass=Environment,
                  gridStateclass=GridStateFromFile,
                  # type of chronics to use. For example GridStateFromFile if forecasts are not used, or GridStateFromFileWithForecasts otherwise
@@ -239,8 +222,12 @@ class Runner(object):
                  gridStateclass_kwargs={},
                  voltageControlerClass=ControlVoltageFromFile,
                  thermal_limit_a=None,
-                 max_iter=-1
-                 ):
+                 max_iter=-1,
+                 other_rewards={},
+                 opponent_action_class=DontAct,
+                 opponent_class=BaseOpponent,
+                 opponent_init_budget=0,
+                 grid_layout=None):
         """
         Initialize the Runner.
 
@@ -314,8 +301,8 @@ class Runner(object):
                 "Parameter \"actionClass\" used to build the Runner should be a type (a class) and not an object "
                 "(an instance of a class). It is currently \"{}\"".format(
                     type(actionClass)))
-        if not issubclass(actionClass, Action):
-            raise RuntimeError("Impossible to create a runner without an action class derived from grid2op.Action. "
+        if not issubclass(actionClass, BaseAction):
+            raise RuntimeError("Impossible to create a runner without an action class derived from grid2op.BaseAction. "
                                "Please modify \"actionClass\" parameter.")
         self.actionClass = actionClass
 
@@ -324,9 +311,9 @@ class Runner(object):
                 "Parameter \"observationClass\" used to build the Runner should be a type (a class) and not an object "
                 "(an instance of a class). It is currently \"{}\"".format(
                     type(observationClass)))
-        if not issubclass(observationClass, Observation):
+        if not issubclass(observationClass, BaseObservation):
             raise RuntimeError("Impossible to create a runner without an observation class derived from "
-                               "grid2op.Observation. Please modify \"observationClass\" parameter.")
+                               "grid2op.BaseObservation. Please modify \"observationClass\" parameter.")
         self.observationClass = observationClass
 
         if not isinstance(rewardClass, type):
@@ -335,9 +322,9 @@ class Runner(object):
                 "(an instance of a class). It is currently \"{}\"".format(
                     type(rewardClass)))
     
-        if not issubclass(rewardClass, Reward):
+        if not issubclass(rewardClass, BaseReward):
             raise RuntimeError("Impossible to create a runner without an observation class derived from "
-                               "grid2op.Reward. Please modify \"rewardClass\" parameter.")
+                               "grid2op.BaseReward. Please modify \"rewardClass\" parameter.")
         self.rewardClass = rewardClass
 
         if not isinstance(gridStateclass, type):
@@ -355,10 +342,10 @@ class Runner(object):
                 "Parameter \"legalActClass\" used to build the Runner should be a type (a class) and not an object "
                 "(an instance of a class). It is currently \"{}\"".format(
                     type(legalActClass)))
-        if not issubclass(legalActClass, LegalAction):
+        if not issubclass(legalActClass, BaseRules):
 
             raise RuntimeError("Impossible to create a runner without a class defining legal actions derived "
-                               "from grid2op.LegalAction. Please modify \"legalActClass\" parameter.")
+                               "from grid2op.BaseRules. Please modify \"legalActClass\" parameter.")
         self.legalActClass = legalActClass
 
         if not isinstance(backendClass, type):
@@ -371,6 +358,7 @@ class Runner(object):
                                "Please modify \"backendClass\" parameter.")
         self.backendClass = backendClass
 
+        self.__can_copy_agent = True
         if agentClass is not None:
             if agentInstance is not None:
                 raise RuntimeError("Impossible to build the backend. Only one of AgentClass or agentInstance can be "
@@ -380,19 +368,24 @@ class Runner(object):
                     "Parameter \"agentClass\" used to build the Runner should be a type (a class) and not an object "
                     "(an instance of a class). It is currently \"{}\"".format(
                         type(agentClass)))
-            if not issubclass(agentClass, Agent):
-                raise RuntimeError("Impossible to create a runner without an agent class derived from grid2op.Agent. "
+            if not issubclass(agentClass, BaseAgent):
+                raise RuntimeError("Impossible to create a runner without an agent class derived from grid2op.BaseAgent. "
                                    "Please modify \"agentClass\" parameter.")
             self.agentClass = agentClass
             self._useclass = True
             self.agent = None
         elif agentInstance is not None:
-            if not isinstance(agentInstance, Agent):
-                raise RuntimeError("Impossible to create a runner without an agent class derived from grid2op.Agent. "
+            if not isinstance(agentInstance, BaseAgent):
+                raise RuntimeError("Impossible to create a runner without an agent class derived from grid2op.BaseAgent. "
                                    "Please modify \"agentInstance\" parameter.")
             self.agentClass = None
             self._useclass = False
             self.agent = agentInstance
+            # Test if we can copy the agent for parallel runs
+            try:
+                copy.copy(self.agent)
+            except:
+                self.__can_copy_agent = False
         else:
             raise RuntimeError("Impossible to build the backend. Either AgentClass or agentInstance must be provided "
                                "and both are None.")
@@ -423,7 +416,7 @@ class Runner(object):
         self.gridStateclass_kwargs = gridStateclass_kwargs
         self.max_iter = max_iter
         if max_iter > 0:
-            self.gridStateclass["max_iter"] = max_iter
+            self.gridStateclass_kwargs["max_iter"] = max_iter
         self.chronics_handler = ChronicsHandler(chronicsClass=self.gridStateclass,
                                                 path=self.path_chron,
                                                 **self.gridStateclass_kwargs)
@@ -442,6 +435,25 @@ class Runner(object):
         if not issubclass(voltageControlerClass, ControlVoltageFromFile):
             raise Grid2OpException("Parameter \"voltagecontrolClass\" should derive from \"ControlVoltageFromFile\".")
         self.voltageControlerClass = voltageControlerClass
+        self._other_rewards = other_rewards
+
+        # for opponent (should be defined here) after the initialization of _BasicEnv
+        if not issubclass(opponent_action_class, BaseAction):
+            raise EnvError("Impossible to make an environment with an opponent action class not derived from BaseAction")
+        try:
+            self.opponent_init_budget = float(opponent_init_budget)
+        except Exception as e:
+            raise EnvError("Impossible to convert \"opponent_init_budget\" to a float with error {}".format(e))
+        if self.opponent_init_budget < 0.:
+            raise EnvError("If you want to deactive the opponent, please don't set its budget to a negative number."
+                           "Prefer the use of the DontAct action type (\"opponent_action_class=DontAct\" "
+                           "and / or set its budget to 0.")
+        if not issubclass(opponent_class, BaseOpponent):
+            raise EnvError("Impossible to make an opponent with a type that does not inherit from BaseOpponent.")
+        self.opponent_action_class = opponent_action_class
+        self.opponent_class = opponent_class
+        self.opponent_init_budget = opponent_init_budget
+        self.grid_layout = grid_layout
 
     def _new_env(self, chronics_handler, backend, parameters):
         res = self.envClass(init_grid_path=self.init_grid_path,
@@ -453,15 +465,25 @@ class Runner(object):
                             observationClass=self.observationClass,
                             rewardClass=self.rewardClass,
                             legalActClass=self.legalActClass,
-                            voltagecontrolerClass=self.voltageControlerClass)
+                            voltagecontrolerClass=self.voltageControlerClass,
+                            other_rewards=self._other_rewards,
+                            opponent_action_class=self.opponent_action_class,
+                            opponent_class=self.opponent_class,
+                            opponent_init_budget=self.opponent_init_budget)
 
         if self.thermal_limit_a is not None:
             res.set_thermal_limit(self.thermal_limit_a)
 
+        if self.grid_layout is not None:
+            res.attach_layout(self.grid_layout)
+
         if self._useclass:
             agent = self.agentClass(res.helper_action_player)
         else:
-            agent = self.agent
+            if self.__can_copy_agent:
+                agent = copy.copy(self.agent)
+            else:
+                agent = self.agent
         return res, agent
 
     def init_env(self):
@@ -530,6 +552,8 @@ class Runner(object):
         env.chronics_handler.tell_id(indx-1)
         # the "-1" above is because the environment will be reset. So it will increase id of 1.
         obs = env.reset()
+        # reset the agent
+        agent.reset()
 
         # compute the size and everything if it needs to be stored
         nb_timestep_max = env.chronics_handler.max_timestep()
@@ -567,7 +591,8 @@ class Runner(object):
                               action_space=env.action_space,
                               helper_action_env=env.helper_action_env,
                               path_save=path_save, disc_lines_templ=disc_lines_templ,
-                              logger=logger, name=env.chronics_handler.get_name())
+                              logger=logger, name=env.chronics_handler.get_name(),
+                              other_rewards=[])
 
         episode.set_parameters(env)
 
@@ -596,7 +621,7 @@ class Runner(object):
         episode.set_meta(env, time_step, cum_reward)
 
         li_text = ["Env: {:.2f}s", "\t - apply act {:.2f}s", "\t - run pf: {:.2f}s",
-                   "\t - env update + observation: {:.2f}s", "Agent: {:.2f}s", "Total time: {:.2f}s",
+                   "\t - env update + observation: {:.2f}s", "BaseAgent: {:.2f}s", "Total time: {:.2f}s",
                    "Cumulative reward: {:1f}"]
         msg_ = "\n".join(li_text)
         logger.info(msg_.format(
@@ -679,7 +704,7 @@ class Runner(object):
 
               - "id_chron" unique identifier of the episode
               - "name_chron" name of chronics
-              - "cum_reward" the cumulative reward obtained by the :attr:`Runner.Agent` on this episode i
+              - "cum_reward" the cumulative reward obtained by the :attr:`Runner.BaseAgent` on this episode i
               - "nb_time_step": the number of time steps played in this episode.
               - "max_ts" : the maximum number of time steps of the chronics
 
@@ -720,6 +745,8 @@ class Runner(object):
         """
         This method will run in parrallel, independantly the nb_episode over nb_process.
 
+        In case the agent cannot be cloned using `copy.copy`: nb_process is set to 1
+
         Note that it restarts completely the :attr:`Runner.backend` and :attr:`Runner.env` if the computation
         is actually performed with more than 1 cores (nb_process > 1)
 
@@ -748,7 +775,7 @@ class Runner(object):
 
               - "i" unique identifier of the episode (compared to :func:`Runner.run_sequential`, the elements of the
                 returned list are not necessarily sorted by this value)
-              - "cum_reward" the cumulative reward obtained by the :attr:`Runner.Agent` on this episode i
+              - "cum_reward" the cumulative reward obtained by the :attr:`Runner.BaseAgent` on this episode i
               - "nb_time_step": the number of time steps played in this episode.
               - "max_ts" : the maximum number of time steps of the chronics
 
@@ -756,7 +783,7 @@ class Runner(object):
         if nb_process <= 0:
             raise RuntimeError(
                 "Runner: you need at least 1 process to run episodes")
-        if nb_process == 1:
+        if nb_process == 1 or self.__can_copy_agent is False:
             warnings.warn(
                 "Runner.run_parrallel: number of process set to 1. Failing back into sequential mod.")
             return [self.run_sequential(nb_episode, path_save=path_save)]
@@ -803,7 +830,7 @@ class Runner(object):
 
               - "i" unique identifier of the episode (compared to :func:`Runner.run_sequential`, the elements of the
                 returned list are not necessarily sorted by this value)
-              - "cum_reward" the cumulative reward obtained by the :attr:`Runner.Agent` on this episode i
+              - "cum_reward" the cumulative reward obtained by the :attr:`Runner.BaseAgent` on this episode i
               - "nb_time_step": the number of time steps played in this episode.
 
         """
