@@ -1,8 +1,4 @@
-try:
-    from .Exceptions import Grid2OpException
-except (ModuleNotFoundError, ImportError):
-    from Exceptions import Grid2OpException
-
+from grid2op.Exceptions import Grid2OpException
 
 def extract_from_dict(dict_, key, converter):
     if not key in dict_:
@@ -46,4 +42,4 @@ class _FakePbar(object):
         return self
 
     def __exit__(self, exc_type, exc_val, exc_tb):
-        return False
\ No newline at end of file
+        return False
